from pygov_br.django_apps.base import BaseDataImporter
from pygov_br.django_apps.camara_deputados import models
from pygov_br.camara_deputados import cd


class SessionImporter(BaseDataImporter):

    field_relation = {
        'code': 'codigo',
        'date': 'data',
        'session_type': 'tipo',
        'number': 'numero',
    }

    def get_data(self):
        return cd.sessions.speeches('15/01/2016', '15/12/2016')

    def get_model(self):
        return models.Session

    def clean_session_type(self, data):
        data = data.replace(' - CD', '')
        return models.SessionType.objects.get_or_create(description=data)[0]


class SpeechImporter(BaseDataImporter):

    field_relation = {
        'initial_time': 'horaInicioDiscurso',
        'insertion_number': 'numeroInsercao',
        'quarter_number': 'numeroQuarto',
        'order': 'orador',
        'summary': 'sumario',
        'indexes': 'txtIndexacao',
        'author': 'orador',
        'session': 'codigoSessao',
        'session_phase': 'faseSessao',
    }

    def get_data(self):
        sessions = cd.sessions.speeches('15/01/2016', '15/12/2016')
        speeches_list = []
        for session in sessions:
            session_code = session['codigo']
            phases = session['fasesSessao']['faseSessao']
            if isinstance(phases, list):
                for phase in phases:
                    speeches = phase['discursos']['discurso']
                    for speech in speeches:
                        speech['codigoSessao'] = session_code
                        speech['faseSessao'] = {
                            'codigo': phase['codigo'],
                            'descricao': phase['descricao']
                        }
            else:
                speeches = phases['discursos']['discurso']
                if isinstance(speeches, list):
                    for speech in speeches:
                        speech['codigoSessao'] = session_code
                        speech['faseSessao'] = {
                            'codigo': phases['codigo'],
                            'descricao': phases['descricao']
                        }
                    speeches_list += speeches
                else:
                    speech = speeches
                    speech['codigoSessao'] = session_code
                    speech['faseSessao'] = {
                        'codigo': phases['codigo'],
                        'descricao': phases['descricao']
                    }
                    speeches_list.append(speech)
        return speeches_list

    def get_model(self):
        return models.Speech

    def clean_author(self, data):
        deputy_name = data['nome'].split(' (')
        try:
            return models.Deputy.objects.get(parliamentary_name=deputy_name[0])
        except models.Deputy.DoesNotExist:
            return None

    def clean_session(self, data):
        return models.Session.objects.get(code=data)

    def clean_session_phase(self, data):
        return models.SessionPhase.objects.get_or_create(
            code=data['codigo'],
            description=data['descricao']
        )[0]

    def clean_order(self, data):
        return data['numero']

    # def after_save_object(self, obj):
    #     speech = cd.sessions.full_speech(
    #         obj.session.code, obj.order,
    #         obj.quarter_number, obj.insertion_number
    #     )
    #     if speech['discurso'] is not None:
    #         obj.full_text = speech['discurso']
    #         obj.save()



# -*- coding: utf-8 -*-
from distutils.util import strtobool
from xml.etree.ElementTree import fromstring, ElementTree
from pygov_br.exceptions import ClientError, ClientServerError
from datetime import datetime
from inspect import isclass
import pytz
import logging
import requests
import sys

if sys.version_info < (3, 0):  # pragma: no cover
    from urlparse import urljoin
else:  # pragma: no cover
    from urllib.parse import urljoin

log = logging.getLogger('pygov_br.client')


class ClientWrapper(object):
    """Base class to client wrapper."""

    def __init__(self, **kwargs):
        for key in kwargs.keys():
            if isclass(kwargs[key]):
                value = kwargs[key]()
            else:
                value = kwargs[key]
            setattr(self, key, value)


class Client(object):
    """Base class to interact with API."""

    def __init__(self, host, timeout=None):
        self.host = host
        self.timeout = timeout

    def _get(self, path, **kwargs):
        return self._request('GET', path, kwargs)

    def _request(self, verb, path, params):
        host = params.pop('host', None)
        if host:
            url = urljoin(host, path)
        else:
            url = urljoin(self.host, path)

        response = requests.request(verb, url, params=params,
                                    timeout=self.timeout)
        log.debug('Response [{0}]: {1}'.format(response.status_code,
                                               repr(response.text)))

        if not response.ok:
            msg = "[{0}]: {1}".format(response.status_code, response.reason)
            if 400 <= response.status_code < 500:
                raise ClientError(msg, response=response)
            else:
                raise ClientServerError(msg + response.text, response=response)

        return response.text

    def _xml_attributes_to_list(self, xml_string, xml_tag):
        element_list = []
        element_tree = ElementTree(fromstring(xml_string.encode('utf-8')))
        for element in element_tree.findall(xml_tag):
            element_list.append(element.attrib)
        return element_list

    def _tree_attributes_to_list(self, element_tree, parent):
        elements = element_tree.find(parent)
        elements_list = []
        for child in elements.getchildren():
            elements_list.append(child.attrib)
        return elements_list

    def _xml_to_dict(self, xml_string):
        etree = ElementTree(fromstring(xml_string.encode('utf-8')))
        return self._make_dict_from_tree(etree.getroot())

    def _make_dict_from_tree(self, element_tree):
        def internal_iter(tree, accum):
            if tree is None:
                return accum

            if tree.getchildren():
                accum[tree.tag] = {}
                for each in tree.getchildren():
                    result = internal_iter(each, {})
                    if each.tag in accum[tree.tag]:
                        if not isinstance(accum[tree.tag][each.tag], list):
                            accum[tree.tag][each.tag] = [
                                accum[tree.tag][each.tag]
                            ]
                        accum[tree.tag][each.tag].append(result[each.tag])
                    else:
                        accum[tree.tag].update(result)
            else:
                accum[tree.tag] = tree.text

            return accum

        return internal_iter(element_tree, {})

    def _safe(self, element):
        if isinstance(element, list):
            safe_element = self._safe_list(element)
        elif isinstance(element, dict):
            safe_element = self._safe_dict(element)
        else:
            safe_element = self._safe_element(element)
        return safe_element

    def _safe_dict(self, dictionary):
        for key in dictionary.keys():
            dictionary[key] = self._safe_element(dictionary[key])

        return dictionary

    def _safe_element(self, element):
        if self._is_digit(element):
            safe_element = int(element)
        elif self._is_float(element):
            safe_element = float(element)
        elif self._is_bool(element):
            safe_element = bool(strtobool(element))
        elif isinstance(element, dict):
            safe_element = self._safe_dict(element)
        elif isinstance(element, list):
            safe_element = self._safe_list(element)
        else:
            safe_element = self._to_date_or_default(element)
        return safe_element

    def _safe_list(self, data_list):
        for index, element in enumerate(data_list):
            data_list[index] = self._safe_element(element)
        return data_list

    def _is_float(self, string):
        try:
            float(string)
            return True
        except (ValueError, TypeError):
            return False

    def _is_digit(self, string):
        try:
            int(string)
            return True
        except (ValueError, TypeError):
            return False

    def _is_bool(self, string):
        try:
            strtobool(string)
            return True
        except (ValueError, TypeError, AttributeError):
            return False

    def _to_date_or_default(self, string):
        final_value = None
        date_formats = {
            '%d/%m/%Y': 'date',
            '%d/%m/%Y %H:%M:%S': None,
            '%d/%m/%Y %H:%M': None,
            '%H:%M': 'time',
            '%H:%M:%S': 'time'
        }
        for date_format in date_formats.keys():
            try:
                final_value = datetime.strptime(string, date_format)
                tz = pytz.timezone('America/Sao_Paulo')
                final_value = tz.localize(final_value)
                if date_formats[date_format]:
                    get_date_type = getattr(final_value,
                                            date_formats[date_format])
                    final_value = get_date_type()
                break
            except (ValueError, TypeError):
                final_value = string.strip() if string else None
        return final_value



# -*- coding: utf-8 -*-
from requests.exceptions import HTTPError


class ClientError(HTTPError):
    """ A generic error while attempting to communicate with the server """


class ClientServerError(ClientError):
    """ The server encountered an error while processing the request """


class MissingParameterError(Exception):
    """ Some parameter is missing """



"""
Pyth -- Python text markup and conversion
"""
from __future__ import absolute_import

import os.path

__version__ = '0.6.0'

writerMap = {
    '.rtf': 'pyth.plugins.rtf15.writer.Rtf15Writer',
    '.html': 'pyth.plugins.xhtml.writer.XHTMLWriter',
    '.xhtml': 'pyth.plugins.xhtml.writer.XHTMLWriter',
    '.txt': 'pyth.plugins.plaintext.writer.PlaintextWriter',
    '.pdf': 'pyth.plugins.pdf.writer.PDFWriter',
}


mimeMap = {
    '.rtf': 'application/rtf',
    '.html': 'text/html',
    '.xhtml': 'application/xhtml+xml',
    '.txt': 'text/plain',
}



def write(doc, filename):
    ext = os.path.splitext(filename)[1]
    writer = namedObject(writerMap[ext])
    buff = writer.write(doc)
    buff.seek(0)
    return (buff, mimeMap[ext])


# Stolen from twisted.python.reflect

def namedModule(name):
    """Return a module given its name."""
    topLevel = __import__(name)
    packages = name.split(".")[1:]
    m = topLevel
    for p in packages:
        m = getattr(m, p)
    return m


def namedObject(name):
    """Get a fully named module-global object.
    """
    classSplit = name.split('.')
    module = namedModule('.'.join(classSplit[:-1]))
    return getattr(module, classSplit[-1])
    



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2016-12-20 22:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Deputy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('outbuilding', models.IntegerField()),
                ('budget_id', models.IntegerField(null=True)),
                ('condition', models.CharField(max_length=50)),
                ('email', models.EmailField(max_length=254)),
                ('phone', models.CharField(max_length=15)),
                ('cabinet', models.IntegerField()),
                ('parliamentary_id', models.IntegerField()),
                ('enrollment_id', models.IntegerField()),
                ('name', models.CharField(max_length=150)),
                ('parliamentary_name', models.CharField(max_length=100)),
                ('gender', models.CharField(max_length=50)),
                ('region', models.CharField(max_length=2)),
                ('photo_url', models.URLField()),
                ('parliamentary_seat_leader', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name_plural': 'Deputies',
                'verbose_name': 'Deputy',
            },
        ),
        migrations.CreateModel(
            name='LegislativeBody',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('initials', models.CharField(max_length=10)),
                ('description', models.TextField()),
            ],
            options={
                'verbose_name_plural': 'Legislative Bodies',
                'verbose_name': 'Legislative Body',
            },
        ),
        migrations.CreateModel(
            name='LegislativeBodyMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('deputy', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='comissions', to='camara_deputados.Deputy')),
                ('legislative_body', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.LegislativeBody')),
            ],
            options={
                'verbose_name_plural': 'Legislative Body Members',
                'verbose_name': 'Legislative Body Member',
            },
        ),
        migrations.CreateModel(
            name='LegislativeBodyRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name_plural': 'Legislative Body Roles',
                'verbose_name': 'Legislative Body Role',
            },
        ),
        migrations.CreateModel(
            name='LegislativeBodyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name_plural': 'Legislative Body Types',
                'verbose_name': 'Legislative Body Type',
            },
        ),
        migrations.CreateModel(
            name='ParliamentarySeat',
            fields=[
                ('name', models.CharField(max_length=150)),
                ('initials', models.CharField(max_length=20, primary_key=True, serialize=False)),
            ],
            options={
                'verbose_name_plural': 'Parliamentary Seats',
                'verbose_name': 'Parliamentary Seat',
            },
        ),
        migrations.CreateModel(
            name='Party',
            fields=[
                ('creation_date', models.DateField(null=True)),
                ('extinction_date', models.DateField(null=True)),
                ('name', models.CharField(max_length=100)),
                ('initials', models.CharField(max_length=20, primary_key=True, serialize=False)),
            ],
            options={
                'verbose_name_plural': 'Parties',
                'verbose_name': 'Party',
            },
        ),
        migrations.CreateModel(
            name='PartyBloc',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_date', models.DateField(null=True)),
                ('extinction_date', models.DateField(null=True)),
                ('name', models.CharField(max_length=100)),
                ('initials', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name_plural': 'Party Blocs',
                'verbose_name': 'Party Bloc',
            },
        ),
        migrations.CreateModel(
            name='PartyBlocMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('adhesion_date', models.DateField(null=True)),
                ('shutdown_date', models.DateField(null=True)),
                ('party', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='party_blocs', to='camara_deputados.Party')),
                ('party_bloc', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.PartyBloc')),
            ],
            options={
                'verbose_name_plural': 'Party Bloc Members',
                'verbose_name': 'Party Bloc Member',
            },
        ),
        migrations.CreateModel(
            name='Session',
            fields=[
                ('code', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('date', models.DateField()),
                ('number', models.IntegerField()),
            ],
            options={
                'verbose_name_plural': 'Sessions',
                'verbose_name': 'Session',
            },
        ),
        migrations.CreateModel(
            name='SessionPhase',
            fields=[
                ('code', models.CharField(max_length=50, primary_key=True, serialize=False)),
                ('description', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name_plural': 'Session Phases',
                'verbose_name': 'Session Phase',
            },
        ),
        migrations.CreateModel(
            name='SessionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name_plural': 'Session Types',
                'verbose_name': 'SessionType',
            },
        ),
        migrations.CreateModel(
            name='Speech',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('initial_time', models.DateTimeField()),
                ('insertion_number', models.IntegerField()),
                ('quarter_number', models.IntegerField()),
                ('order', models.IntegerField()),
                ('full_text', models.TextField()),
                ('summary', models.TextField(null=True)),
                ('indexes', models.TextField(null=True)),
                ('author', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='speeches', to='camara_deputados.Deputy')),
                ('session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='speeches', to='camara_deputados.Session')),
                ('session_phase', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='speeches', to='camara_deputados.SessionPhase')),
            ],
            options={
                'verbose_name_plural': 'Speeches',
                'verbose_name': 'Speech',
            },
        ),
        migrations.AddField(
            model_name='session',
            name='session_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.SessionType'),
        ),
        migrations.AddField(
            model_name='partybloc',
            name='members',
            field=models.ManyToManyField(through='camara_deputados.PartyBlocMember', to='camara_deputados.Party'),
        ),
        migrations.AddField(
            model_name='legislativebodymember',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.LegislativeBodyRole'),
        ),
        migrations.AddField(
            model_name='legislativebody',
            name='legislative_body_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='camara_deputados.LegislativeBodyType'),
        ),
        migrations.AddField(
            model_name='legislativebody',
            name='members',
            field=models.ManyToManyField(through='camara_deputados.LegislativeBodyMember', to='camara_deputados.Deputy'),
        ),
        migrations.AddField(
            model_name='deputy',
            name='parliamentary_seat',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='members', to='camara_deputados.ParliamentarySeat'),
        ),
        migrations.AddField(
            model_name='deputy',
            name='party',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='members', to='camara_deputados.Party'),
        ),
    ]



# -*- coding: utf8 -*-
#
# This file were created by Python Boilerplate. Use boilerplate to start simple
# usable and best-practices compliant Python projects.
#
# Learn more about it at: http://github.com/fabiommendes/python-boilerplate/
#

import os
import sys
from setuptools import setup, find_packages


# Meta information
name = 'pygov-br-mj'
project = 'pygov_br'
author = 'Matheus Fernandes'
version = open('VERSION').read().strip()
dirname = os.path.dirname(__file__)


# Save version and author to __meta__.py
with open(os.path.join(dirname, 'src', project, '__meta__.py'), 'w') as F:
    F.write('__version__ = %r\n__author__ = %r\n' % (version, author))

INSTALL_DEPS = [
    'requests>=2.11.1',
    'beautifulsoup4',
    'click',
    'pytz'
]

if sys.version_info < (2, 7):
    INSTALL_DEPS.append('importlib')

setup(
    # Basic info
    name=name,
    version=version,
    author=author,
    author_email='matheus.souza.fernandes@gmail.com',
    url='',
    description='A short description for your project.',
    long_description=open('README.rst').read(),

    # Classifiers (see https://pypi.python.org/pypi?%3Aaction=list_classifiers)
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
    ],

    # Packages and depencies
    package_dir={'': 'src'},
    packages=find_packages('src'),
    install_requires=INSTALL_DEPS,
    extras_require={
        'dev': [
            'mock',
            'responses',
            'pytest',
            'pytest-cov',
            'coveralls',
            'flake8',
            'manuel',
        ],
    },

    # Scripts
    entry_points={
        'console_scripts': ['pygov-br = pygov_br.__main__:main'],
    },

    # Other configurations
    zip_safe=False,
    platforms='any',
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-07-20 23:47
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('camara_deputados', '0005_auto_20170719_2330'),
    ]

    operations = [
        migrations.AlterField(
            model_name='proposal',
            name='author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='proposals', to='camara_deputados.Deputy'),
        ),
    ]



# -*- coding: utf-8 -*-
from pygov_br.camara_deputados import cd
from datetime import datetime
import responses


@responses.activate
def test_legislative_body_all():
    xml_response = """
    <orgaos>
        <orgao sigla="CAPADR "/>
        <orgao sigla="CCJC "/>
    </orgaos>
    """
    expected_list = [{'sigla': 'CAPADR'}, {'sigla': 'CCJC'}]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ObterOrgaos',
        body=xml_response, status=200)
    assert cd.legislative_bodies.all() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_roles():
    xml_response = """
    <cargosOrgaos>
        <cargo id="1" descricao="Presidente"/>
        <cargo id="2" descricao="1º Vice-Presidente"/>
    </cargosOrgaos>
    """
    expected_list = [{'id': 1, 'descricao': 'Presidente'},
                     {'id': 2, 'descricao': u'1\xc2\xba Vice-Presidente'}]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/'
        'ListarCargosOrgaosLegislativosCD',
        body=xml_response, status=200)
    assert cd.legislative_bodies.roles() == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_members():
    xml_response = """
    <orgao nome="Comissão de Defesa do Consumidor">
        <membros>
            <Presidente>
                <nome>Marco Tebaldi</nome>
            </Presidente>
            <PrimeiroVice-Presidente>
                <nome>Nelson Marchezan Junior</nome>
            </PrimeiroVice-Presidente>
            <SegundoVice-Presidente>
                <nome>Marcos Rotta</nome>
            </SegundoVice-Presidente>
            <TerceiroVice-Presidente>
                <nome>Maria Helena</nome>
            </TerceiroVice-Presidente>
            <membro>
                <nome>Antônio Jácome</nome>
            </membro>
            <membro>
                <nome>Celso Russomanno</nome>
            </membro>
        </membros>
    </orgao>
    """
    expected_dict = {
        'Presidente': {'nome': 'Marco Tebaldi'},
        'PrimeiroVice-Presidente': {'nome': 'Nelson Marchezan Junior'},
        'SegundoVice-Presidente': {'nome': 'Marcos Rotta'},
        'TerceiroVice-Presidente': {'nome': 'Maria Helena'},
        'membro': [{'nome': u'Ant\xc3\xb4nio J\xc3\xa1come'},
                   {'nome': 'Celso Russomanno'}]
    }
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ObterMembrosOrgao',
        body=xml_response, status=200)
    assert cd.legislative_bodies.members(1) == expected_dict
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_schedule():
    xml_response = """
    <pauta orgao="CDC" dataInicial="01/01/2012" dataFinal="30/04/2012">
        <reuniao>
            <comissao>CDC</comissao>
            <proposicoes>
                <proposicao>
                    <sigla>PL 1762/2011</sigla>
                </proposicao>
                <proposicao>
                    <sigla>PL 1390/2012</sigla>
                </proposicao>
            </proposicoes>
        </reuniao>
        <reuniao>
            <comissao>CDC</comissao>
            <proposicoes>
                <proposicao>
                    <sigla>PL 1762/2011</sigla>
                </proposicao>
                <proposicao>
                    <sigla>PL 1390/2012</sigla>
                </proposicao>
            </proposicoes>
        </reuniao>
    </pauta>
    """
    expected_list = [
        {'comissao': 'CDC',
         'proposicoes': {'proposicao': [{'sigla': 'PL 1762/2011'},
                                        {'sigla': 'PL 1390/2012'}]}},
        {'comissao': 'CDC',
         'proposicoes': {'proposicao': [{'sigla': 'PL 1762/2011'},
                                        {'sigla': 'PL 1390/2012'}]}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ObterPauta',
        body=xml_response, status=200)
    result_list = cd.legislative_bodies.schedule(1, '10/10/2010', '10/10/2010')
    assert result_list == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_schedule_dict():
    xml_response = """
    <pauta orgao="CDC" dataInicial="01/01/2012" dataFinal="30/04/2012">
        <reuniao>
            <comissao>CDC</comissao>
            <proposicoes>
                <proposicao>
                    <sigla>PL 1762/2011</sigla>
                </proposicao>
                <proposicao>
                    <sigla>PL 1390/2012</sigla>
                </proposicao>
            </proposicoes>
        </reuniao>
    </pauta>
    """
    expected_list = [
        {'comissao': 'CDC',
         'proposicoes': {'proposicao': [{'sigla': 'PL 1762/2011'},
                                        {'sigla': 'PL 1390/2012'}]}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ObterPauta',
        body=xml_response, status=200)
    result_list = cd.legislative_bodies.schedule(1, '10/10/2010', '10/10/2010')
    assert result_list == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_schedule_with_datetime():
    xml_response = """
    <pauta orgao="CDC" dataInicial="01/01/2012" dataFinal="30/04/2012">
        <reuniao>
            <comissao>CDC</comissao>
            <proposicoes>
                <proposicao>
                    <sigla>PL 1762/2011</sigla>
                </proposicao>
                <proposicao>
                    <sigla>PL 1390/2012</sigla>
                </proposicao>
            </proposicoes>
        </reuniao>
        <reuniao>
            <comissao>CDC</comissao>
            <proposicoes>
                <proposicao>
                    <sigla>PL 1762/2011</sigla>
                </proposicao>
                <proposicao>
                    <sigla>PL 1390/2012</sigla>
                </proposicao>
            </proposicoes>
        </reuniao>
    </pauta>
    """
    expected_list = [
        {'comissao': 'CDC',
         'proposicoes': {'proposicao': [{'sigla': 'PL 1762/2011'},
                                        {'sigla': 'PL 1390/2012'}]}},
        {'comissao': 'CDC',
         'proposicoes': {'proposicao': [{'sigla': 'PL 1762/2011'},
                                        {'sigla': 'PL 1390/2012'}]}}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ObterPauta',
        body=xml_response, status=200)
    result_list = cd.legislative_bodies.schedule(1, datetime.now(),
                                                 datetime.now())
    assert result_list == expected_list
    assert len(responses.calls) == 1


@responses.activate
def test_legislative_body_types():
    xml_response = """
    <tipoOrgaos>
        <tipoOrgao id="81000" descricao="Bancada"/>
        <tipoOrgao id="1000" descricao="Sociedade Civil"/>
    </tipoOrgaos>
    """
    expected_list = [
        {'id': 81000, 'descricao': 'Bancada'},
        {'id': 1000, 'descricao': 'Sociedade Civil'},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/SitCamaraWS/Orgaos.asmx/ListarTiposOrgaos',
        body=xml_response, status=200)
    assert cd.legislative_bodies.types() == expected_list
    assert len(responses.calls) == 1



"""
Render documents as reStructuredText.
"""
from __future__ import absolute_import

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter

from cStringIO import StringIO


class RSTWriter(PythWriter):

    @classmethod
    def write(klass, document, target=None):
        if target is None:
            target = StringIO()

        writer = RSTWriter(document, target)
        return writer.go()

    def __init__(self, doc, target):
        self.document = doc
        self.target = target
        self.indent = -1
        self.paragraphDispatch = {document.List: self.list,
                                  document.Paragraph: self.paragraph}

    def go(self):
        for (i, paragraph) in enumerate(self.document.content):
            handler = self.paragraphDispatch[paragraph.__class__]
            handler(paragraph)
            self.target.write("\n")

        # Heh heh, remove final paragraph spacing
        self.target.seek(-2, 1)
        self.target.truncate()

        return self.target

    def text(self, text):
        """
        process a pyth text and return the formatted string
        """
        ret = u"".join(text.content)
        if 'url' in text.properties:
            return u"`%s`_" % ret
        if 'bold' in text.properties:
            return u"**%s**" % ret
        if 'italic' in text.properties:
            return u"*%s*" % ret
        if 'sub' in text.properties:
            return ur"\ :sub:`%s`\ " % ret
        if 'super' in text.properties:
            return ur"\ :sup:`%s`\ " % ret
        return ret

    def paragraph(self, paragraph, prefix=""):
        """
        process a pyth paragraph into the target
        """
        content = []
        for text in paragraph.content:
            content.append(self.text(text))
        content = u"".join(content).encode("utf-8")

        for line in content.split("\n"):
            self.target.write("  " * self.indent)
            self.target.write(prefix)
            self.target.write(line)
            self.target.write("\n")
            if prefix:
                prefix = "  "

        # handle the links
        if any('url' in text.properties for text in paragraph.content):
            self.target.write("\n")
            for text in paragraph.content:
                if 'url' in text.properties:
                    string = u"".join(text.content)
                    url = text.properties['url']
                    self.target.write(".. _%s: %s\n" % (string, url))

    def list(self, list, prefix=None):
        """
        Process a pyth list into the target
        """
        self.indent += 1
        for (i, entry) in enumerate(list.content):
            for (j, paragraph) in enumerate(entry.content):
                prefix = "- " if j == 0 else "  "
                handler = self.paragraphDispatch[paragraph.__class__]
                handler(paragraph, prefix)
                self.target.write("\n")
        self.indent -= 1



# -*- coding: utf-8 -*-
from pygov_br.camara_deputados import cd
from datetime import datetime
import responses


@responses.activate
def test_session_speeches():
    xml_response = """
    <sessoesDiscursos>
        <sessao>
            <codigo>321.2.54.O</codigo>
            <fasesSessao>
                <faseSessao>
                    <descricao>Homenagem</descricao>
                    <discursos>
                        <discurso>
                            <orador>
                                <nome>MAURO BENEVIDES (PRESIDENTE)</nome>
                            </orador>
                            <txtIndexacao>RECURSOS APOIO.</txtIndexacao>
                        </discurso>
                        <discurso>
                            <orador>
                                <nome>MAURO BENEVIDES (PRESIDENTE)</nome>
                            </orador>
                            <txtIndexacao>APOIO RECURSOS.</txtIndexacao>
                        </discurso>
                    </discursos>
                </faseSessao>
            </fasesSessao>
        </sessao>
    </sessoesDiscursos>
    """
    expected_list = [
        {'codigo': '321.2.54.O',
         'fasesSessao': {'faseSessao': {
             'descricao': 'Homenagem',
             'discursos': {'discurso': [
                 {'orador': {'nome': 'MAURO BENEVIDES (PRESIDENTE)'},
                  'txtIndexacao': 'RECURSOS APOIO.'},
                 {'orador': {'nome': 'MAURO BENEVIDES (PRESIDENTE)'},
                  'txtIndexacao': 'APOIO RECURSOS.'}
             ]}}
         }}
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarDiscursosPlenario',
        body=xml_response, status=200)
    assert cd.sessions.speeches('10/10/2010', '10/10/2010') == expected_list
    datetime_result = cd.sessions.speeches(
        datetime(2010, 10, 10), datetime(2010, 10, 10))
    assert datetime_result == expected_list
    assert len(responses.calls) == 2


@responses.activate
def test_session_speeches_multiple_sessions():
    xml_response = """
    <sessoesDiscursos>
        <sessao>
            <codigo>321.2.54.O</codigo>
            <fasesSessao>
                <faseSessao>
                    <descricao>Homenagem</descricao>
                    <discursos>
                        <discurso>
                            <orador>
                                <nome>MAURO BENEVIDES (PRESIDENTE)</nome>
                            </orador>
                            <txtIndexacao>RECURSOS APOIO.</txtIndexacao>
                        </discurso>
                    </discursos>
                </faseSessao>
            </fasesSessao>
        </sessao>
        <sessao>
            <codigo>321.2.54.O</codigo>
            <fasesSessao>
                <faseSessao>
                    <descricao>Homenagem</descricao>
                    <discursos>
                        <discurso>
                            <orador>
                                <nome>MAURO BENEVIDES (PRESIDENTE)</nome>
                            </orador>
                            <txtIndexacao>RECURSOS APOIO.</txtIndexacao>
                        </discurso>
                    </discursos>
                </faseSessao>
            </fasesSessao>
        </sessao>
    </sessoesDiscursos>
    """
    expected_list = [
        {'codigo': '321.2.54.O',
         'fasesSessao': {'faseSessao': {
             'descricao': 'Homenagem',
             'discursos': {
                 'discurso':
                 {'orador': {'nome': 'MAURO BENEVIDES (PRESIDENTE)'},
                  'txtIndexacao': 'RECURSOS APOIO.'}
             }}
         }},
        {'codigo': '321.2.54.O',
         'fasesSessao': {'faseSessao': {
             'descricao': 'Homenagem',
             'discursos': {
                 'discurso':
                 {'orador': {'nome': 'MAURO BENEVIDES (PRESIDENTE)'},
                  'txtIndexacao': 'RECURSOS APOIO.'}
             }}
         }},
    ]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarDiscursosPlenario',
        body=xml_response, status=200)
    assert cd.sessions.speeches('10/10/2010', '10/10/2010') == expected_list
    datetime_result = cd.sessions.speeches(
        datetime(2010, 10, 10), datetime(2010, 10, 10))
    assert datetime_result == expected_list
    assert len(responses.calls) == 2


@responses.activate
def test_session_full_speech():
    xml_response = """
    <sessao>
        <nome>DUDIMAR PAXIUBA</nome>
        <discursoRTFBase64>
            e1xydGYxXGFuc2lcZGVmZjANCntcY29sb3J0Ymw7XHJlZDBcZ3JlZW4wXGJsdWU
            wO1xyZWQyNTVcZ3JlZW4wXGJsdWUwO30NClRoaXMgbGluZSBpcyB0aGUgZGVmYX
            VsdCBjb2xvclxsaW5lfQ==
        </discursoRTFBase64>
    </sessao>
    """
    expected_dict = {'nome': 'DUDIMAR PAXIUBA',
                     'discurso': 'This line is the default color'}
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'obterInteiroTeorDiscursosPlenario',
        body=xml_response, status=200)
    assert cd.sessions.full_speech('200.10.3', 1, 1, 1) == expected_dict
    assert len(responses.calls) == 1


@responses.activate
def test_session_frequency():
    xml_response = """
    <dia>
        <legislatura>54</legislatura>
        <parlamentares>
            <parlamentar>
                <nomeParlamentar>Berinho Bantim</nomeParlamentar>
                <sessoesDia>
                    <sessaoDia>
                        <descricao>ORDINÁRIA 073</descricao>
                    </sessaoDia>
                    <sessaoDia>
                        <descricao>EXTRAORDINÁRIA 074</descricao>
                    </sessaoDia>
                </sessoesDia>
            </parlamentar>
        </parlamentares>
    </dia>
    """
    expected_dict = {
        'legislatura': 54,
        'parlamentares': {'parlamentar': {
            'nomeParlamentar': 'Berinho Bantim',
            'sessoesDia': {'sessaoDia': [
                {'descricao': u'ORDINÃ\x81RIA 073'},
                {'descricao': u'EXTRAORDINÃ\x81RIA 074'}
            ]}}}
    }
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarPresencasDia',
        body=xml_response, status=200)
    assert cd.sessions.frequency('10/10/2010') == expected_dict
    assert cd.sessions.frequency(datetime(2010, 10, 10)) == expected_dict
    assert len(responses.calls) == 2


@responses.activate
def test_session_status():
    xml_response = """
    <situacaoReuniaoSessao>
        <situacaoReuniao id="1" descricao="Não Confirmada "/>
        <situacaoReuniao id="2" descricao="Convocada "/>
    </situacaoReuniaoSessao>
    """
    expected_list = [{'id': 1, 'descricao': u'NÃ£o Confirmada'},
                     {'id': 2, 'descricao': 'Convocada'}]
    responses.add(
        responses.GET,
        'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        'ListarSituacoesReuniaoSessao',
        body=xml_response, status=200)
    assert cd.sessions.status() == expected_list
    assert len(responses.calls) == 1



"""
Render documents as XHTML fragments
"""
from __future__ import absolute_import

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythWriter

import six


_tagNames = {
    'bold': 'strong',
    'italic': 'em',
    'underline': 'u', # ?
}


class XHTMLWriter(PythWriter):

    @classmethod
    def write(klass, document, target=None, cssClasses=True, pretty=False):
        if target is None:
            target = six.BytesIO()

        writer = XHTMLWriter(document, target, cssClasses, pretty)
        final = writer.go()
        final.seek(0)

        # Doesn't work all that well -- appends an <?xml ...> tag,
        # and puts line breaks in unusual places for HTML.
        #if pretty:
        #    content = final.read()
        #    final.seek(0)
        #    from xml.dom.ext import PrettyPrint
        #    from xml.dom.ext.reader.Sax import FromXml
        #    PrettyPrint(FromXml(content), final)
        #    final.seek(0)

        return final


    def __init__(self, doc, target, cssClasses=True, pretty=False):
        self.document = doc
        self.target = target
        self.cssClasses = cssClasses
        self.pretty = pretty
        self.paragraphDispatch = {
            document.List: self._list,
            document.Paragraph: self._paragraph
        }


    def go(self):

        self.listLevel = -1

        tag = Tag("div")

        for element in self.document.content:
            handler = self.paragraphDispatch[element.__class__]
            tag.content.extend(handler(element))

        tag.render(self.target)
        return self.target


    def _paragraph(self, paragraph):
        p = Tag("p")
        for text in paragraph.content:
            p.content.append(self._text(text))

        if self.pretty:
            return [_prettyBreak, p, _prettyBreak]
        else:
            return [p]


    def _list(self, lst):
        self.listLevel += 1

        ul = Tag("ul")

        if self.cssClasses:
            ul.attrs['class'] = 'pyth_list_%s' % self.listLevel

        for entry in lst.content:
            li = Tag("li")
            for element in entry.content:
                handler = self.paragraphDispatch[element.__class__]
                li.content.extend(handler(element))
            ul.content.append(li)

        self.listLevel -= 1

        return [ul]


    def _text(self, text):
        if 'url' in text.properties:
            tag = Tag("a")
            tag.attrs['href'] = text.properties['url']
        else:
            tag = Tag(None)

        current = tag

        for prop in ('bold', 'italic', 'underline'):
            if prop in text.properties:
                newTag = Tag(_tagNames[prop])
                current.content.append(newTag)
                current = newTag

        for prop in ('sub', 'super'):
            if prop in text.properties:
                if current.tag is None:
                    newTag = Tag("span")
                    current.content.append(newTag)
                    current = newTag
                current.attrs['style'] = "vertical-align: %s; font-size: smaller" % prop

        current.content.append(u"".join(text.content))

        return tag



_prettyBreak = object()


class Tag(object):

    def __init__(self, tag, attrs=None, content=None):
        self.tag = tag
        self.attrs = attrs or {}
        self.content = content or []


    def render(self, target):

        if self.tag is not None:
            attrString = self.attrString()
            if attrString:
                attrString = " " + attrString
            target.write(('<%s%s>' % (self.tag, attrString)).encode("utf-8"))

        for c in self.content:
            if isinstance(c, Tag):
                c.render(target)
            elif c is _prettyBreak:
                target.write(b'\n')
            else:
                if type(c) != six.text_type:
                    c = six.text_type(c)
                target.write(quoteText(c).encode("utf-8").replace(b'\n', b'<br />'))

        if self.tag is not None:
            target.write(('</%s>' % self.tag).encode("utf-8"))


    def attrString(self):
        return " ".join(
            '%s="%s"' % (k, quoteAttr(v))
            for (k, v) in six.iteritems(self.attrs))


    def __repr__(self):
        return "T(%s)[%s]" % (self.tag, repr(self.content))



def quoteText(text):
    return text.replace(
        u"&", u"&amp;").replace(
        u"<", u"&lt;").replace(
        u">", u"&gt;")


def quoteAttr(text):
    return quoteText(text).replace(
        u'"', u"&quot;").replace(
        u"'", u"&apos;")


def write_html_file(filename, bytescontent, print_msg=True):
    # used in examples and in tests
    with open(filename, "wb") as out:
        out.write(b"<!DOCTYPE html>\n")
        out.write(b"<html><head><meta charset='utf-8' /></head><body>\n")
        out.write(bytescontent)
        out.write(b"\n</body></html>")
    if print_msg:
        print("##### wrote RTF as XHTML to", filename)



from django.db import transaction
from pygov_br.django_apps.base import BaseDataImporter
from pygov_br.django_apps.camara_deputados import models
from pygov_br.camara_deputados import cd


class ProposalTypeImporter(BaseDataImporter):

    field_relation = {
        "id": "id",
        "is_active": "ativa",
        "description": "descricao",
        "gender": "genero",
        "initials": "tipoSigla",
    }

    def get_model(self):
        return models.ProposalType

    def get_data(self):
        return cd.proposals.types()


class ProposalImporter(BaseDataImporter):

    field_relation = {
        "id": "id",
        "year": "ano",
        "number": "numero",
        "submission_date": "datApresentacao",
        "name": "nome",
        "summary": "txtEmenta",
        "summary_explanation": "txtExplicacaoEmenta",
        "proposal_type": "tipoProposicao",
    }

    def get_model(self):
        return models.Proposal

    def get_data(self):
        return cd.proposals.filter(proposal_type='PL', year=2016)

    def clean_author(self, data):
        if data['idecadastro']:
            return models.Deputy.objects.get(enrollment_id=data['idecadastro'])
        else:
            return None

    def clean_proposal_type(self, data):
        return models.ProposalType.objects.get(initials=data['sigla'])

    @transaction.atomic
    def after_save_object(self, obj):
        extra_info = cd.proposals.get(proposal_type=obj.proposal_type.initials,
                                      proposal_number=obj.number,
                                      year=obj.year)
        obj.indexes = extra_info['Indexacao']
        obj.link = extra_info['LinkInteiroTeor']
        if extra_info['ideCadastro']:
            try:
                author = models.Deputy.objects.get(
                    id=extra_info['ideCadastro']
                )
                obj.author = author
            except models.Deputy.DoesNotExist:
                print('nao existe', extra_info['ideCadastro'])

        obj.save()



# -*- coding: utf-8 -*-
from pygov_br.base import Client
from pygov_br.vendor.pyth.plugins.rtf15.reader import Rtf15Reader
from pygov_br.vendor.pyth.plugins.xhtml.writer import XHTMLWriter
from base64 import b64decode
from datetime import datetime
from bs4 import BeautifulSoup
from tempfile import TemporaryFile


class SessionClient(Client):

    def __init__(self):
        host = 'http://www.camara.leg.br/sitcamaraws/SessoesReunioes.asmx/'
        super(SessionClient, self).__init__(host)

    def speeches(self, initial_date, final_date, session_id='',
                 parliamentary_name='', party_initials='', region=''):
        """Fetch all speeches in a period.

        Args:
            initial_date (str or datetime): Initial date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            final_date (str or datetime): Final date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            session_id (int, optional): Session identifier. Defaults to ''.
            parliamentary_name (str, optional): A parliamentary name. Defaults
                to ''.
            party_initials (str, optional): Party identifier initials. Defaults
                to ''.
            region (str, optional): Brazilian region identifier initials.
                Defaults to ''.

        Returns:
            list: A list of sessions that contains speeches made by deputies on
            Plenary of Câmara dos Deputados in a specific period. For example::

                [{'codigo': '320.2.54.O',
                  'data': datetime.date(2012, 11, 23),
                  'fasesSessao': {'faseSessao': {'codigo': 'PE',
                    'descricao': 'Pequeno Expediente',
                    'discursos': {'discurso': [{
                       'horaInicioDiscurso':
                            datetime.datetime(2012, 11, 23, 9, 18),
                       'numeroInsercao': 0,
                       'numeroQuarto': 3,
                       'orador': {'nome': 'MAURO BENEVIDES',
                        'numero': 2,
                        'partido': 'PMDB',
                        'uf': 'CE'},
                       'sumario': 'Presença da Presidenta Dilma Rousseff em ' \
                                  'Fortaleza, Estado do Ceará, para a ' \
                                  'inauguração do novo Estádio Castelão e da' \
                                  'Zona de Processamento de Exportação - ZPE' \
                                  'Liberação da área destinada à construção ' \
                                  'da Refinaria Premium II no Estado pela '\
                                  'PETROBRAS.',
                       'txtIndexacao': 'ESTÁDIO, FUTEBOL, CIDADE, FORTALEZA,' \
                                       'CE, INAUGURAÇÃO, PRESENÇA, DILMA ' \
                                       'ROUSSEFF, PRESIDENTE DA REPÚBLICA, ' \
                                       CONFIRMAÇÃO, APOIO.'},
                  'numero': 2,
                  'tipo': 'Ordinária - CD'}, ...]

        """
        if isinstance(initial_date, datetime):
            initial_date = initial_date.strftime('%d/%m/%Y')
        if isinstance(final_date, datetime):
            final_date = final_date.strftime('%d/%m/%Y')

        path = "ListarDiscursosPlenario?dataIni={0}&dataFim={1}&" \
               "codigoSessao={2}&parteNomeParlamentar={3}&" \
               "siglaPartido={4}&siglaUF={5}"
        xml_response = self._get(path.format(
            initial_date, final_date, session_id, parliamentary_name,
            party_initials, region
        ))
        xml_dict = self._xml_to_dict(xml_response)
        list_response = xml_dict['sessoesDiscursos']['sessao']

        if isinstance(list_response, dict):
            list_response = [list_response]

        return self._safe(list_response)

    def full_speech(self, session_id, speaker_number, quarter, insertion):
        """Fetch full content of a speech.

        All parameter can be caught with the `speeches` method.

        Args:
            session_id (str): Session identifier.
            speaker_number (int): Speaker identifier.
            quarter (int): Shorthand fraction number that identifies the
                beginning of the speech.
            insertion (int): Shorthand insertion number that identifies the
                beginning of the speech.

        Returns:
            dict: The full content of a specified speech with the deputy
            informations and datetime from the speech.

            'DiscursoRTF' key is a byte string that contains the full speech in
            Rich Text Format.

            Example::

                {'discursoRTF': b"{\\rtf1\\ansi\\ansicpg1252\\deff0" \
                                 "\\deflang1046\\deflangfe1046\\deftab709" \
                                 "{\\fonttbl{\\f0\\fswiss\\fprq2\\fcharset0 " \
                                 "Arial;}}\r\n\\viewkind4\\uc1\\pard\\sl480" \
                                 "\\slmult1\\qj\\f0\\fs24\\tab\\b O SR. " \
                                 "DUDIMAR PAXIUBA \\b0 (PSDB-PA. Sem " \
                                 "revis\\'e3o do orador.) - Sr. Presidente, "\
                                 "Sras. e Srs. Parlamentares, ocupo esta " \
                                 "tribuna para parabenizar a torcida " ... },
                 'horaInicioDiscurso': datetime.datetime(2013, 3, 4, 14, 33),
                 'nome': 'DUDIMAR PAXIUBA',
                 'partido': 'PSDB',
                 'uf': 'PA'}

        """
        path = "obterInteiroTeorDiscursosPlenario?codSessao={0}" \
               "&numOrador={1}&numQuarto={2}&numInsercao={3}"
        xml_response = self._get(path.format(session_id, speaker_number,
                                             quarter, insertion))
        xml_dict = self._xml_to_dict(xml_response)['sessao']
        speech = b64decode(xml_dict.pop('discursoRTFBase64'))
        xml_dict['discurso'] = self._extract_text_from_rtf(speech)
        return self._safe(xml_dict)

    def _extract_text_from_rtf(self, rtf_text):
        try:
            temp_file = TemporaryFile()
            temp_file.write(rtf_text)
            doc = Rtf15Reader.read(temp_file)
            html = XHTMLWriter.write(doc, pretty=True).read()
            cleantext = BeautifulSoup(html, "html.parser").text.strip()
            return cleantext
        except (UnicodeDecodeError, TypeError):
            return None

    def frequency(self, session_date, legislature='', deputy_enrollment_id='',
                  party_initials='', region=''):
        """Fetch sessions ferquency.

        List the parliamentary frequency on sessions in specific date.

        Args:
            session_date (str or datetime): Initial date of period. If `str`,
                must be in the format: `dd/mm/yyyy`.
            lesgislature (int, optional): To specify an legislature. Defaults
                to ''.
            deputy_enrollment_id (int, optional): Deputy enrollment identifier.
                Defaults to ''.
            party_initials (str, optional): Party identifier initials. Defaults
                to ''.
            region (str, optional): Brazilian region identifier initials.
                Defaults to ''.

        Returns:
            dict: Returns a dict with all session that occurred in the
            specified date and the frequency of parliamentarians in each
            session. For example::

                {'data': datetime.date(2012, 4, 10),
                 'legislatura': 54,
                 'parlamentares': {'parlamentar': {'carteiraParlamentar': 1,
                   'descricaoFrequenciaDia': 'Presença',
                   'justificativa': None,
                   'nomeParlamentar': 'Berinho Bantim-PSDB/RR',
                   'presencaExterna': 0,
                   'sessoesDia': {'sessaoDia': [{
                      'descricao': 'ORDINÁRIA Nº 073 - 10/04/2012',
                      'frequencia': 'Presença',
                      'inicio': datetime.datetime(2012, 4, 10, 14, 0, 14)},
                     {'descricao': 'EXTRAORDINÁRIA Nº 074 - 10/04/2012',
                      'frequencia': 'Presença',
                      'inicio': datetime.datetime(2012, 4, 10, 18, 46, 53)
                      }, ...]},
                   'siglaPartido': 'PSDB',
                   'siglaUF': 'RR'}, ...},
                 'qtdeSessoesDia': 2}

        """
        if isinstance(session_date, datetime):
            session_date = session_date.strftime('%d/%m/%Y')

        path = "ListarPresencasDia?data={0}&numLegislatura={1}&" \
               "numMatriculaParlamentar={2}&siglaPartido={3}&siglaUF={4}"
        xml_response = self._get(path.format(
            session_date, legislature, deputy_enrollment_id,
            party_initials, region
        ))
        xml_dict = self._xml_to_dict(xml_response)
        return self._safe(xml_dict['dia'])

    def status(self):
        """Fetch all status for comissions and sessions.

        Returns:
            list: A list of possible situations for commissions and sessions.
            For example::

                [{'descricao': 'Encerrada(Comunicado)', 'id': 9}, ...]

        """
        xml_response = self._get('ListarSituacoesReuniaoSessao')
        list_response = self._xml_attributes_to_list(xml_response,
                                                     'situacaoReuniao')
        return self._safe(list_response)



"""
Read documents from xhtml
"""
from __future__ import absolute_import

import BeautifulSoup

from pygov_br.vendor.pyth import document
from pygov_br.vendor.pyth.format import PythReader
from pygov_br.vendor.pyth.plugins.xhtml.css import CSS
import six


class XHTMLReader(PythReader):

    @classmethod
    def read(self, source, css_source=None, encoding="utf-8", link_callback=None):
        reader = XHTMLReader(source, css_source, encoding, link_callback)
        return reader.go()

    def __init__(self, source, css_source=None, encoding="utf-8", link_callback=None):
        self.source = source
        self.css_source = css_source
        self.encoding = encoding
        self.link_callback = link_callback

    def go(self):
        soup = BeautifulSoup.BeautifulSoup(self.source,
                                           convertEntities=BeautifulSoup.BeautifulSoup.HTML_ENTITIES,
                                           fromEncoding=self.encoding,
                                           smartQuotesTo=None)
        # Make sure the document content doesn't use multi-lines
        soup = self.format(soup)
        doc = document.Document()
        if self.css_source:
            self.css = CSS(self.css_source)
        else:
            self.css = CSS()    # empty css
        self.process_into(soup, doc)
        return doc

    def format(self, soup):
        """format a BeautifulSoup document

        This will transform the block elements content from
        multi-lines text into single line.

        This allow us to avoid having to deal with further text
        rendering once this step has been done.
        """
        # Remove all the newline characters before a closing tag.
        for node in soup.findAll(text=True):
            if node.rstrip(" ").endswith("\n"):
                node.replaceWith(node.rstrip(" ").rstrip("\n"))
        # Join the block elements lines into a single long line
        for tag in ['p', 'li']:
            for node in soup.findAll(tag):
                text = six.text_type(node)
                lines = [x.strip() for x in text.splitlines()]
                text = ' '.join(lines)
                node.replaceWith(BeautifulSoup.BeautifulSoup(text))
        soup = BeautifulSoup.BeautifulSoup(six.text_type(soup))
        # replace all <br/> tag by newline character
        for node in soup.findAll('br'):
            node.replaceWith("\n")
        soup = BeautifulSoup.BeautifulSoup(six.text_type(soup))
        return soup

    def is_bold(self, node):
        """
        Return true if the BeautifulSoup node needs to be rendered as
        bold.
        """
        return (node.findParent(['b', 'strong']) is not None or
                self.css.is_bold(node))

    def is_italic(self, node):
        """
        Return true if the BeautifulSoup node needs to be rendered as
        italic.
        """
        return (node.findParent(['em', 'i']) is not None
                or self.css.is_italic(node))

    def is_sub(self, node):
        """
        Return true if the BeautifulSoup node needs to be rendered as
        sub.
        """
        return (node.findParent(['sub']) is not None
                or self.css.is_sub(node))

    def is_super(self, node):
        """
        Return true if the BeautifulSoup node needs to be rendered as
        super.
        """
        return (node.findParent(['sup']) is not None
                or self.css.is_super(node))

    def url(self, node):
        """
        return the url of a BeautifulSoup node or None if there is no
        url.
        """
        a_node = node.findParent('a')
        if not a_node:
            return None

        if self.link_callback is None:
            return a_node.get('href')
        else:
            return self.link_callback(a_node.get('href'))

    def process_text(self, node):
        """
        Return a pyth Text object from a BeautifulSoup node or None if
        the text is empty.
        """
        text = node.string.strip()
        if not text:
            return

        # Set all the properties
        properties=dict()
        if self.is_bold(node):
            properties['bold'] = True
        if self.is_italic(node):
            properties['italic'] = True
        if self.url(node):
            properties['url'] = self.url(node)
        if self.is_sub(node):
            properties['sub'] = True
        if self.is_super(node):
            properties['super'] = True

        content=[node.string]

        return document.Text(properties, content)

    def process_into(self, node, obj):
        """
        Process a BeautifulSoup node and fill its elements into a pyth
        base object.
        """
        if isinstance(node, BeautifulSoup.NavigableString):
            text = self.process_text(node)
            if text:
                obj.append(text)
            return
        if node.name == 'p':
            # add a new paragraph into the pyth object
            new_obj = document.Paragraph()
            obj.append(new_obj)
            obj = new_obj
        elif node.name == 'ul':
            # add a new list
            new_obj = document.List()
            obj.append(new_obj)
            obj = new_obj
        elif node.name == 'li':
            # add a new list entry
            new_obj = document.ListEntry()
            obj.append(new_obj)
            obj = new_obj
        for child in node:
            self.process_into(child, obj)
